%{
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
extern int yyparse( void );
extern int yylex( void );
extern int yylex_destroy( void );
extern int yylineno;


static int expr_result = 0;
static int yyerrors   = 0;


void yyerror(const char *str)
{
        fprintf(stderr,"line %d: error: %s\n", yylineno, str);
	yyerrors++;
}
 
int yywrap( void )
{
        return 1;
} 

int main( int argc, char **argv )
{
        if( yyparse() == 0 && yyerrors == 0 )
	{
		printf( "parsing succeeded\n" );
		printf( "result: %d\n", expr_result );
	} else
	{
		fprintf( stderr, "parsing failed with %d errors\n", yyerrors );
	}
        yylex_destroy();
	return 0;
} 
%}

/* some tokens and grammar rules have an associated value (many do not):
 * the %union lists all possible "shapes" of associated values.
 */
%union { int n; }

/* list the tokens, and for those with associated data (NUMBER)
 * which named field in the union is associated with it.
 */
%token PLUS MINUS MUL DIV MOD OPEN CLOSE TOKERR
%token <n> NUMBER

/* some grammatical rules have an associated value (many do not):
 * %type associates which named field in the union is associated
 * with which grammatical rules
 */
%type <n> expr term factor

/*
 * the parser that Yacc builds will attempt to match the WHOLE
 * INPUT to this start rule: called "here" (the name doesn't matter).
 * so this means: recognise an expression followed by nothing else.
 */
%start here

%%
here         :  expr            { expr_result = $1; }
             ;

expr         : expr PLUS term   { $$ = $1 + $3; }
             | expr MINUS term  { $$ = $1 - $3; }
             | term             { $$ = $1;      }
             ;

term         : term MUL factor  { $$ = $1 * $3; }
             | term DIV factor  { $$ = $1 / $3; }
             | term MOD factor  { $$ = $1 % $3; }
             | factor           { $$ = $1;      }
             ;

factor       : NUMBER           { $$ = $1; }
             | OPEN expr CLOSE  { $$ = $2; }
             ;
