/*
 * named constants stored in a hash
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include <longhash.h>

#include "consthash.h"


extern void yyerror(const char *str);
extern bool verbose;


/* the constant hash */
static longhash consthash = NULL;


/* helper function: print (k,v) pair */
static void printkv( FILE *out, longhashkey k, longhashvalue v )
{
	fprintf( out, "\t  %.20s: %ld\n", k, v );
}


void init_consthash( int argc, char **argv )
{
	consthash = longhashCreate( &printkv );
	int i;
	for( i=1; i<argc; i++ )
	{
		char key[8];
		sprintf( key, "arg%d", i );
		longhashSet( consthash, key, atol(argv[i]) );
	}
	long power;
	for( i=0, power=1; i<32; i++, power<<=1 )
	{
		char key[8];
		sprintf( key, "t%d", i );
		longhashSet( consthash, key, power );
	}
	longhashSet( consthash, "howmanyroads", 42 );
	longhashSet( consthash, "one",   1 );
	longhashSet( consthash, "two",   2 );
	longhashSet( consthash, "three", 3 );
	longhashSet( consthash, "four",  4 );
	longhashSet( consthash, "five",  5 );
	longhashSet( consthash, "six",   6 );
	longhashSet( consthash, "seven", 7 );
	longhashSet( consthash, "eight", 8 );
	longhashSet( consthash, "nine",  9 );
	longhashSet( consthash, "ten",   10 );
	if( verbose )
	{
		fprintf( stderr, "\n\tInitialized consthash containing:\n" );
		longhashDump( stderr, consthash );
	}
}


void destroy_consthash( void )
{
	longhashFree( consthash );
}


int lookup_const( char *constname )
{
	longhashvalue v = longhashFind( consthash, constname );
	if( v == -1 )
	{
		char mesg[1024];
		sprintf( mesg, "undefined constant '%s'", constname );
		yyerror( mesg );
		return -1;
	}
	return (int)v;
}


void foreach_constant( constkvcb cb, void *extra )
{
	longhashForeach( consthash, (longhashforeachcb)cb, extra );
}
