/*
 * eval.c:	expression evaluation...
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "consthash.h"
#include "types.h"
#include "binfuncs.h"
#include "eval.h"


// a binary evaluator is a pointer to a function like intplus or intminus
// etc that takes two integers a and b, and returns the sum, difference or
// whatever of a and b as the result.

typedef int (*binaryevaluator)( int a, int b );

// CAREFUL: this table of binary evaluators MUST BE IN THE SAME ORDER as the
// arithop's in types.in for this to work..

static binaryevaluator opfunc[] = {
	intplus,		// arithop_is_plus,
	intminus,		// arithop_is_minus,
	inttimes,		// arithop_is_times,
	intdivide,		// arithop_is_divide,
	intmod,			// arithop_is_mod,
};


// evaluate an Abstract Syntax Tree "expr", returning it's value.
int eval_expr( expr e )
{
	if( expr_kind(e) == expr_is_num )
	{
		int n;
		get_expr_num( e, &n );
		return n;
	}
	if( expr_kind(e) == expr_is_id )
	{
		string s;
		get_expr_id( e, &s );
		return lookup_const( s );
	}
	// must be binop()
	assert( expr_kind(e) == expr_is_binop );
	expr l, r; arithop op;
	get_expr_binop( e, &l, &op, &r );
	int lv = eval_expr( l );
	int rv = eval_expr( r );

	// now need to combine lv and rv under the operator op
	binaryevaluator f = opfunc[op];
	return (*f)( lv, rv );
}
