#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include "consthash.h"
#include "types.h"


extern int yyparse( void );
extern int yylex_destroy (void);
extern int yyerrors;
extern program ast;


bool verbose     = false;


int main( int argc, char **argv )
{
	int opt;

        while ((opt = getopt(argc, argv, "v")) != -1) {
		switch (opt) {
		case 'v':
			verbose = true;
			break;
		default: /* '?' */
			fprintf(stderr, "Usage: %s [-v] [arg1 [arg2..]]\n",
				argv[0]);
			exit(1);
		}
	}

	int i, j;
	for( i=optind, j=1; i<argc; i++, j++ )
	{
		argv[j] = argv[i];
	}
	argc -= optind-1;

	init_consthash( argc, argv );
	if( verbose )
	{
		fprintf( stderr, "\tStarting parsing\n" );
	}
        if( yyparse() == 0 && yyerrors == 0 )
	{
		fprintf( stderr, "Parsing succeeded\n" );
		if( verbose ) printf( "AST is:\n" );
		print_program( stdout, ast );
	} else
	{
		fprintf( stderr, "Parsing failed with %d errors\n", yyerrors );
		exit(1);
	}
	destroy_consthash();
	yylex_destroy();
	free_program( ast );

	return 0;
} 
