/*
 *	C codegenerator for THS
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <assert.h>

#include "consthash.h"
#include "types.h"
#include "funchash.h"
#include "codegen.h"


extern bool verbose;
extern bool expand_constants;


static void write_c_main( FILE *out, expr e )
{
	printf( "int main( void )\n{\n\t" );
	char *mesg = "printf( %cResult: %%d\\n%c, ";
	printf( mesg, '"', '"' );
	print_expr( stdout, e );
	printf( ");\n\treturn 0;\n}\n\n" );
}


static int nfuncs = 0;
static char **farray = NULL;


static void countfuncs( char *name, fdefn defn )
{
	nfuncs++;
}


static void copyintofarray( char *name, fdefn defn )
{
	static int pos = 0;
	assert( pos < nfuncs );
	farray[pos++] = name;
}


static int comparestring(const void *a, const void *b)
{
	return strcmp( *((char **)a), *((char **)(b)) );
}


/*
 * let's sort the functions so we can generate neat prototypes
 * and definitions.  set nfuncs, create farray[] dynarray,
 * populate it, and sort it.
 */
void sort_functions( void )
{
	nfuncs = 0;
	foreach_function( &countfuncs );
	if( verbose )
	{
		fprintf( stderr, "sort_functions: there are %d functions\n",
			nfuncs );
	}
	farray = (char **)malloc( nfuncs * sizeof(char *) );
	foreach_function( &copyintofarray );
	qsort( farray, nfuncs, sizeof(char *), &comparestring );
}


void free_farray( void )
{
	if( farray != NULL )
	{
		free( farray );
		farray = NULL;
	}
}


/*
 * generate prototypes using the sorted farray/nfuncs data
 */
void mk_prototypes( void )
{
	int i;
	if( nfuncs == 0 ) return;

	printf( "/* forward prototype declarations */\n" );
	for( i=0; i<nfuncs; i++ )
	{
		printf( "int %s( int );\n", farray[i] );
	}
	printf( "\n" );
}


static void cdefine_oneconst( char *name, int n, void *extra )
{
	FILE *out = (FILE *)extra;
	fprintf( out, "#define %-20s %u\n", name, n );
}


void write_c_program( FILE *out, expr e )
{
	printf( "\n#include <stdio.h>\n" );
	printf( "#include <stdlib.h>\n\n" );

	printf( "#define FALLOFF(f,n) {fprintf( stderr, "
	        "%cfallen off the end of function "
		"%c #f %c( %%d )\\n%c, n );exit(1);}\n\n",
			'"', '"', '"', '"' );
	if( ! expand_constants )
	{
		printf( "/* named constants */\n" );
		foreach_constant( &cdefine_oneconst, stdout );
		printf( "\n\n" );
	}

	sort_functions();
	mk_prototypes();

	print_funchash( stdout );
	write_c_main( out, e );
}
