/*
 * testintstack: a few unit tests
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include <testutils.h>

#include "intstack.h"

int main( void )
{
	intstack s = intstack_new();
	testbool( intstack_isempty(s), "[] is empty" );
	testbool( ! intstack_isfull(s), "[] is not full" );
	testint( intstack_nelements(s), 0, "[] has 0 elements" );

	intstack_push( s, 10 );
	testbool( ! intstack_isempty(s), "[10] is not empty" );
	testbool( ! intstack_isfull(s), "[10] is not full" );
	testint( intstack_nelements(s), 1, "[10] has 1 element" );

	intstack_push( s, 20 );
	testbool( ! intstack_isempty(s), "[20,10] is not empty" );
	testbool( ! intstack_isfull(s), "[20,10] is not full" );
	testint( intstack_nelements(s), 2, "[20,10] has 2 elements" );

	printf( "max depth stack is: " );
	intstack_print( s, stdout );
	printf( "\n" );

	int el = intstack_pop( s );
	testint( el, 20, "pop([20,10]) is 20" );
	testbool( ! intstack_isempty(s), "pop([20,10]) is not empty" );
	testbool( ! intstack_isfull(s), "pop([20,10]) is not full" );

	el = intstack_pop( s );
	testint( el, 10, "pop(pop([20,10])) is 10" );
	testbool( intstack_isempty(s), "pop(pop([20,10])) is empty" );
	testbool( ! intstack_isfull(s), "pop(pop([20,10])) is not full" );

	intstack_destroy(s);
	return 0;
}
