This is a new "combine hashes and sets" example, even simpler than my
dcw-mini-list-flattening example.

It implements a named collection of sets of strings: a hash-of-sets-of-strings,
given that we have already implemented sets-of-strings and
hashes-from-strings-to-generic-pointers (here as set.[ch] and hash.[ch])

This version uses my CM ("C with Modules") tools to generate sets, hashes and
family collection modules from *.cm


# Background...

Suppose we are presented with an input file with any number of lines
of the form:

```
one: a               (meaning that "one is the parent of a")
two: c               (meaning that "two is the parent of c")
one: b               (meaning that "one is the parent of b")
one: c               (meaning that "one is the parent of c")
```

ie. a named parent (eg "one") may have any number of named children (eg "b"),
there may be any number of parents, any number of children per parent,
and the parent,child information may come in any order.

and our goal is to collect all the children belonging to each parent,
producing output showing, for each parent, the complete set of
their children (ideally sorted):

```
one: b,c,d          (one is the parent of b, c and d)
two: c              (two is the parent of c only)
```

Clearly, the core data type here is a hash from parent names (strings) to
sets of strings (the set of children).

We store this as a hash-of-sets-of-strings,
implemented by the hashofsets.[ch] module and unit-tested by "testhos.c".
The main program that transforms the input to the output is "transform.c".


# This version...

This version of the code shows off my new C+NAMESPACES tool,
my existing sets-of-strings module, and my hash-from-strings-to-void*
module, and the app-specific family collection module itself, are all
now defined in .cn files, from which both the .c and the .h are generated.

I realised while building this, as each module contained a %pubfunc called
"free", which gets decorated as "hash_free" or "set_free" or "famcoll_free"
depending on which namespace it's in, but also wants to call the real C
standard library "free" function too, that must NOT be decorated, that it
might be necessary to temporarily, explicitly, disable decoration on a line
by line basis.  So I added "@nodec" [yes, @ not %, easier to handle] as in:

	@nodec free( (void *)f );

which ensures that line is translated as:

	free( (void *)f );


# Building this version

1. build this project via:

```
make
```

2. run it's tests:

```
make test
```

3. run transform on the pc-input file and check for memory leaks:

```
valgrind ./transform < pc-input
```


4. Note that the summarisetests utility here is useful



# Doing transform as a Perl one-liner

Depressingly, after writing nearly 500 lines of C code (not including the
hash and set modules!), it turns out that we could write transform as
a Perl one-liner:

```
perl -MData::Dumper -lane '($p,$c) = split( /:\s*/ ); $h{$p}->{$c}++; END {print Dumper \%h }' < pc-input
```

which produces the output:

```
$VAR1 = {
          'one' => {
                     'c' => 1,
                     'a' => 1,
                     'b' => 1
                   },
          'two' => {
                     'a' => 1,
                     'd' => 1
                   },
          'three' => {
                       'z' => 1
                     }
        };
```

Data::Dumper is a Perl module that prints out Perl data structures in a
rather JSON like format, and the data structure %h is a hash of hashes,
and the inner hashes are "sethashes", i.e. a hash whose values are all 1,
representing a set - the keys of that hash are the members of the set.

so $h[P]->[C] is a boolean expression that represents "is C a child of parent P"

If we want to format the output in the actual desired form, that'd be:

```
perl -lane '($p,$c) = split( /:\s*/ ); $h{$p}->{$c}++; END {foreach $p (sort keys %h) { $c = $h{$p}; $k = join( ",", sort keys %$c ); print "$p: $k" } }' < pc-input
```

Perl is the most amazing text manipulation language I know.  As the Pragmatic
Programmers say:

	Tip 28: Learn a Text Manipulation Language

they recommend Perl, and they're absolutely right:-)
