/*
 * testset.c: test program for the hash-based set module.
 *
 * (C) Duncan C. White, 1996-2017 although it seems longer:-)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>

#include "set.h"



static set s;


void test( set_key k, int expected )
{
	if( set_in( s, k ) )
	{
		if( ! expected )
		{
			printf( "error: '%s' in when it shouldn't be - ", k );
		}
	} else
	{
		if( expected )
		{
			printf( "error: '%s' not in when it should be\n", k );
		}
	}
}


void lookuptest( int x1, int x2, int x3, int x4, int xaard, int xgrumble )
{
	printf( "lookup tests:\n" );
	test( "eeny", x1 );
	test( "meeny", x2 );
	test( "miny", x3 );
	test( "mo", x4 );
	test( "aardvark", xaard );
	test( "grumble", xgrumble );
}


static void myPrint( FILE *out, set_key k )
{
	fprintf( out, "%s,", k );
}


/* 
 * each_cb: print out an item to the FILE *
 *	    hidden in the arg; used in the main
 *	    program to print the hash ourselves.
 */

static void each_cb( set_key k, void *arg )
{
	FILE *out = (FILE *)arg;
	myPrint( out, k );
}


int main( int argc, char **argv )
{
	s = set_create( myPrint );

	if( argc > 1 )
	{
		malloc(strlen(argv[1]));
	}

	set_add( s, "eeny" );
	set_add( s, "meeny" );
	set_add( s, "miny" );
	set_add( s, "mo" );

	printf( "here's the set:\n" );
	set_dump( stdout, s );

	lookuptest( 1, 1, 1, 1, 0, 0 );

	printf( "modify set a bit:\n" );
	set_remove( s, "meeny" );
	set_add( s, "one" );
	set_add( s, "aardvark" );

	printf( "now here's the set:\n" );
	set_dump( stdout, s );

	lookuptest( 1, 0, 1, 1, 1, 0 );

	printf( "\nprint the set out using an explicit foreach loop:\n{" );
	set_foreach( s, &each_cb, stdout );
	printf( "}\n" );

	printf( "\nfree the set\n" );
	set_free( s );

	return 0;
}
